//
//    File: sys.twn.h
//    Date: 16.11.2011
// Version: 1.17
//
// Purpose:
//
// Define system functions and constants for the transponder reader
// family TWN3.
//
// V1.00: Initial release
// V1.01: Support IndiTag
//        Support MultiISO
// V1.02: iClass: Function for reading application data
//        iClass: Function for generic access to the reader module
//        Mifare and MultiISO: Common functions for module communication
// V1.03: MultiISO: Additional tag type ISO14443B
// V1.04: Inditag: New function 
// V1.05: Support for crypto functions
// V1.06: New function HostSendCmd
// V1.07: New beep type "no beep"
// V1.08: New function GetRandomBytes
// V1.09: New function HostSendNumber
// V1.10: New tag type Felica
// V1.11: New function ConvertDigitsToBinary
// V1.12: New function ConvertBinaryToASCII
// V1.13: New function SetOutputs and GetOutputs
// V1.14: Definitions for BEEPLOWLONG, BEEPHIGHLONG, BLINKFAST
//        Update of new versions of device types
// V1.15: New tag types SRX
//        New system function SetModule
// V1.16: New functions LegicSearchDataSegmentByIndex, 
//        LegicSearchDataSegmentByStamp, LegicGetSegmentDataLength
// V1.17: New function CalcCRC8

// General constants for boolean operations
const byte FALSE = 0;
const byte TRUE = 1;

// Enumeration of all system functions
const byte RESET                    = 0;
const byte TAGSEARCH                = 1;
const byte TAGREAD                  = 2;
const byte TAGWRITE                 = 3;
const byte HOSTSENDVERSION          = 4;
const byte HOSTSENDCHAR             = 5;
const byte HOSTSENDHEX              = 6;
const byte HOSTSENDDEC              = 7;
const byte HOSTTESTCMD              = 8;
const byte LEDSET                   = 9;
const byte LEDGET                   = 10;
const byte SETVOLUME                = 11;
const byte BEEP                     = 12;
const byte COMPBITS                 = 13;
const byte COPYBITS                 = 14;
const byte FILLBITS                 = 15;
const byte SWAPBITS                 = 16;
const byte COMPBYTES                = 17;
const byte COPYBYTES                = 18;
const byte FILLBYTES                = 19;
const byte SWAPBYTES                = 20;
const byte SCANHEX                  = 21;
const byte STARTTIMER               = 22;
const byte STOPTIMER                = 23;
const byte TESTTIMER                = 24;
const byte GETCONNECTION            = 25;
const byte GETUSBMODE               = 26;
const byte GETDEVICETYPE            = 27;
const byte LEGICGENERIC             = 28;
const byte MULTI125GENERIC          = 29;
const byte MULTI125SEARCHLOGIN      = 30;
const byte MIFARELOGIN              = 31;
const byte ICLASSGENERIC            = 32;
const byte ICLASSTAGSEARCHAPP       = 33;
const byte MODULESENDCHAR           = 34;
const byte MODULESENDHEXBYTE        = 35;
const byte MODULERECEIVELINE        = 36;
const byte INDITAGSEARCH2           = 37;
const byte XTEAINIT                 = 38;
const byte XTEAENCRYPT              = 39;
const byte XTEADECRYPT              = 40;
const byte HOSTSENDCMD              = 41;
const byte GETRANDOMBYTES           = 42;
const byte HOSTSENDNUMBER           = 43;
const byte CONVERTDIGITSTOBINARY    = 44;
const byte CONVERTBINARYTOASCII     = 45;
const byte OUTPUTSET                = 47;
const byte OUTPUTGET                = 48;
const byte SETMODULE                = 49;
const byte LEGICSEARCHDATASEGMENTBYINDEX = 50;
const byte LEGICSEARCHDATASEGMENTBYSTAMP = 51;
const byte LEGICGETSEGMENTDATALENGTH     = 52;
const byte CALCCRC8                 = 53;

// Restart the script
void Reset() system RESET;

// Transponder operations
// All types of transponders
const byte TAGTYPE_NONE = 0;
const byte TAGTYPE_MIFARE = 1;
const byte TAGTYPE_HIDPROX = 2;
const byte TAGTYPE_LEGIC = 3;
const byte TAGTYPE_EM4102 = 4;
const byte TAGTYPE_HITAG1S = 5;
const byte TAGTYPE_HITAG2 = 6;
const byte TAGTYPE_EM4150 = 7;
const byte TAGTYPE_AT5555 = 8;
const byte TAGTYPE_ISOFDX = 9;
const byte TAGTYPE_EM4026 = 10;
const byte TAGTYPE_HIDICLASS = 20;
const byte TAGTYPE_ISO15693 = 21;
const byte TAGTYPE_INDITAG = 22;
const byte TAGTYPE_ISO14443B = 23;
const byte TAGTYPE_FELICA = 24;
const byte TAGTYPE_SRX = 25;

// Generally available functions
byte TagSearch(byte &IDData,byte &IDBitCnt,byte &TagType) system TAGSEARCH;
byte TagRead(byte Address,byte ByteCnt,byte &Data) system TAGREAD;
byte TagWrite(byte Address,byte ByteCnt,byte &Data) system TAGWRITE;

// Specific to Multi125
byte Multi125SearchLogin(byte &IDData,byte &IDBitCnt,byte &TagType,byte &Secret) system MULTI125SEARCHLOGIN;
byte Multi125Generic(byte &TXData,byte TXCount,byte &RXData,byte &RXCount,byte MaxRXCount,byte Timeout) system MULTI125GENERIC;

// Mifare and MultiISO specific
const byte KEYA = 0;
const byte KEYB = 1;
byte MifareLogin(byte &Secret,byte KeyType, byte Sector) system MIFARELOGIN;

void ModuleSendChar(byte Char) system MODULESENDCHAR;
void ModuleSendHexByte(byte Byte) system MODULESENDHEXBYTE;
byte ModuleReceiveLine(byte &RXData,byte &RXCount,byte MaxRXCount,byte Timeout) system MODULERECEIVELINE;

// Legic specific
byte LegicSearchDataSegmentByIndex(byte StartIndex,byte &FoundIndex) system LEGICSEARCHDATASEGMENTBYINDEX;
byte LegicSearchDataSegmentByStamp(byte StartIndex,byte StampLength,byte &StampData,byte &FoundIndex) system LEGICSEARCHDATASEGMENTBYSTAMP;
byte LegicGetSegmentDataLength(byte &DataLength) system LEGICGETSEGMENTDATALENGTH;
byte LegicGeneric(byte &TXData,byte TXCount,byte &RXData,byte &RXCount,byte MaxRXCount,byte Timeout) system LEGICGENERIC;

// iClass specific
byte IClassGeneric(byte &TXData,byte TXCount,byte &RXData,byte RXCount,byte Timeout) system ICLASSGENERIC;
byte IClassTagSearchApp(byte &AppData,byte &AppBitCnt) system ICLASSTAGSEARCHAPP;

// Inditag specific
byte IndiTagSearch2(byte &IDData,byte &IDBitCnt,byte &TagType) system INDITAGSEARCH2;

// Host communication functions
void HostSendVersion() system HOSTSENDVERSION;
void HostSendChar(byte Char) system HOSTSENDCHAR;
void HostSendHex(byte &Data,byte BitCnt,byte Width) system HOSTSENDHEX;
void HostSendDec(byte &Data,byte BitCnt,byte Width) system HOSTSENDDEC;
void HostSendNumber(byte &Data,byte FirstBit,byte BitCnt,byte Radix,byte MinWidth,byte MaxWidth) system HOSTSENDNUMBER;

byte HostTestCmd(byte &Cmd,byte &CmdLen,byte MaxCmdLen) system HOSTTESTCMD;
byte HostSendCmd(byte &Cmd,byte CmdLen) system HOSTSENDCMD;

// Access to LEDs and Outputs
const byte GREEN = 1;
const byte RED = 2;
const byte OUTPUT0 = 1;
const byte OUTPUT1 = 2;

const byte OFF = 0;
const byte ON = 1;
const byte BLINK = 2;       // Only valid for LEDs
const byte TOGGLE = 3;
const byte BLINKFAST = 4;   // Only valid for LEDs

void LEDSet(byte LEDs,byte Status) system LEDSET;
byte LEDGet(byte LED) system LEDGET;

void OutputSet(byte Outputs,byte Status) system OUTPUTSET;
byte OutputGet(byte Output) system OUTPUTGET;

// Let's make some noise
const byte BEEPLOW = 0;
const byte BEEPHIGH = 1;
const byte BEEPSUCCESS = 2;
const byte BEEPFAILED = 3;
const byte BEEPNONE = 4;
const byte BEEPLOWLONG = 5;
const byte BEEPHIGHLONG = 6;

void SetVolume(byte Volume) system SETVOLUME;
void Beep(byte Type) system BEEP;

// Bit operations
byte CompBits(byte &Data1,byte &Data2,byte BitCount) system COMPBITS;
void CopyBits(byte &DestBits,byte StartDestBit,byte &SourceBits,byte StartSourceBit,byte BitCount) system COPYBITS;
void FillBits(byte &Dest,byte StartBit,byte Value,byte BitCount) system FILLBITS;
void SwapBits(byte &Data,byte StartBit,byte BitCount) system SWAPBITS;
// Byte operations
byte CompBytes(byte &Data1,byte &Data2,byte ByteCount) system COMPBYTES;
void CopyBytes(byte &DestBytes,byte &SourceBytes,byte ByteCount) system COPYBYTES;
void FillBytes(byte &Dest,byte Value,byte ByteCount) system FILLBYTES;
void SwapBytes(byte &Data,byte ByteCount) system SWAPBYTES;
void GetRandomBytes(byte &Data,byte ByteCount) system GETRANDOMBYTES;
void CalcCRC8(byte CRCStart,byte Polynom,byte &Data,byte ByteCount) system CALCCRC8;

byte ScanHex(byte &Data,byte ByteCnt) system SCANHEX;
void ConvertDigitsToBinary(byte &Dest,byte ByteCnt,byte &Source,byte DigitCnt,byte BitsPerDigit,byte Radix) system CONVERTDIGITSTOBINARY;
byte ConvertBinaryToASCII(byte &Dest,byte &Source,byte FirstBit,byte BitCnt,byte Radix,byte MinDigits,byte MaxDigits) system CONVERTBINARYTOASCII;

// Timer functions, there are 4 timers (ID 0-3) available
void StartTimer(byte ID,byte Time) system STARTTIMER;
void StopTimer(byte ID) system STOPTIMER;
byte TestTimer(byte ID) system TESTTIMER;

// Determine the physical(!) type of connection to the host
const byte V24 = 0;
const byte USB = 1;
byte GetConnection() system GETCONNECTION;

// Determine the operation mode of the USB connection
const byte USBVCOM = 0;
const byte USBHID = 1;
byte GetUSBMode() system GETUSBMODE;

// Determine the type of device
const byte DEVTYPE_MULTI125 = 1;
const byte DEVTYPE_MIFARE = 2;
const byte DEVTYPE_HIDPROX = 3;
const byte DEVTYPE_LEGICPRIME = 4;
const byte DEVTYPE_HIDICLASS = 5;
const byte DEVTYPE_LEGICADVANT = 6;
const byte DEVTYPE_INDITAG = 7;
const byte DEVTYPE_MULTIISO = 8;
const byte DEVTYPE_CHIPCARD = 9;
const byte DEVTYPE_LEGICNFC = 10;
const byte DEVTYPE_MIFARENFC = 11;
const byte DEVTYPE_HIDPROX2 = 12;

byte GetDeviceType() system GETDEVICETYPE;

// Crypto functions
void XTEAInit(byte NRounds,byte &Secret) system XTEAINIT;
void XTEAEncrypt(byte &Data) system XTEAENCRYPT;
void XTEADecrypt(byte &Data) system XTEADECRYPT;

// None-specific functions

void SetModule(byte State) system SETMODULE;

// Prototype for the mandatory function 'main'
void main();
